/*****************************************************************************
**
**  File Name:          interface.c
**  Author:             Dave Helland
**  Date:               5/01/04
**
**  Purpose/Description:
**         Provide Win32 / Linux  I/O facilities
**
**  CSC:  MEGALOADER
**
**  Public Data:
**
**  Private Data:
**
**  Functions:
**
**  Version/Revision History:
**
**
**  Version ID   Date      Author        SPCR No.    Description
**
**  Atmel MegasquirtAVR downlaod program
**  0.0    3/21/01   DRH   great portions borrowed PULOAD
**  1.0    5/04/04   DRH   Initial Release
**
**
***************************************************************************/
/*
You might want to put in something to set the baud
rate in the bSetupDevice - just add in the baud rate in the Dcb
structure before you call SetCommState...

In the main routine is a readfile call which returns nb == 1 if a
character was read, otherwise 0.  It is pretty fast.
*/

/*********************************************************/
/*                   I N C L U D E                       */
/*********************************************************/

#include <stdio.h>

#ifdef __WIN32__
#include <windows.h>
#include <conio.h>
#endif

#ifdef  __linux__
#include <time.h>
#include <sys/time.h>
#include <curses.h>
#include "linux_serial.h"
#include "global.h"
#include "linux_screen.h"
#ifndef STDIN_FILENO
  #define STDIN_FILENO 0
#endif
#endif

#define SERIAL_IO 1
/*********************************************************/
/*           P R O T O T Y P E                           */
/*********************************************************/
#include "defines.h"
#include "interface.h"



#ifdef __WIN32__
/*********************************************************/
/*********************************************************/
/*********************************************************/
/*                W I N D O W S   C O D E                */
/*********************************************************/
/*********************************************************/
/*********************************************************/


/*********************************************************/
/*           G L O B A L    V A R I A B L E S            */
/*********************************************************/
HANDLE hDevice;
DCB Dcb;
char sCommName[10];
BOOL bResult;
LPVOID lpMsgBuf;
DWORD lpErrors;                 // pointer to variable to receive error codes
COMSTAT lpStat;                 // pointer to buffer for communications status
static DWORD save_baud_rate;
    //extern char sCommPortName[10];



/*********************************************************/
/*                   C O D E                             */
/*********************************************************/
static void
ErrorString (void)
{
  char lpMsgBuf[512];

  FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (),
                 MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
                 (LPTSTR) & lpMsgBuf, sizeof (lpMsgBuf), NULL);
  fputs (lpMsgBuf, stderr);
}




BOOLEAN
bSetupDevice (HANDLE hDevice)
{
/*
 * this routine sets the device for a 0 millisecond inter-character timeout
 * for read requests.  in order to allow support of non-communications
 * devices, a failure in GetCommTimeouts is ignored and a success indication
 * is returned.
 */
  COMMTIMEOUTS Timeouts;
  DCB Dcb;

  if (GetCommTimeouts (hDevice, &Timeouts)) {
    Timeouts.ReadIntervalTimeout = MAXDWORD;
    Timeouts.ReadTotalTimeoutMultiplier = 0;
    Timeouts.ReadTotalTimeoutConstant = 0;
    if (SetCommTimeouts (hDevice, &Timeouts)) {
      if (GetCommState (hDevice, &Dcb)) {
        // Dcb.fAbortOnError = 1 ;     // set to abort on error
        Dcb.Parity = NOPARITY;
        Dcb.StopBits = ONESTOPBIT;
        Dcb.fParity = FALSE;
        Dcb.fDtrControl = DTR_CONTROL_DISABLE;
        Dcb.fOutxCtsFlow = FALSE;
        Dcb.fOutxDsrFlow = FALSE;
        Dcb.fDsrSensitivity = FALSE;
        Dcb.fOutX = FALSE;
        Dcb.fInX = FALSE;
        Dcb.fNull = FALSE;
        Dcb.fErrorChar = FALSE;
        Dcb.fRtsControl = RTS_CONTROL_DISABLE;
        Dcb.fAbortOnError = FALSE;
        Dcb.ByteSize = 8;
        if (SetCommState (hDevice, &Dcb)) {
          return (TRUE);
        }
      }
    }
    return (FALSE);             /* something bad happened */
  }
  return (TRUE);
}



int
OpenSerialPort (char *sCommPortName)
{
  // int i;
  // char * p;
  // p = &sCommPortName[0];
  // for (i=0; i<10;i++)  sCommName[i] =  *p++;
  strcpy (sCommName, sCommPortName);

  hDevice = CreateFile (sCommName,      // device name
                        GENERIC_WRITE | GENERIC_READ,   // access mode
                        FILE_SHARE_READ | FILE_SHARE_WRITE,     // share mode
                        NULL,   // default security descriptor
                        OPEN_EXISTING,  // device must exist already!
                        0,      // no overlapped I/O
                        (HANDLE) 0      // no template file
    );

  if (hDevice != INVALID_HANDLE_VALUE) {
    bSetupDevice (hDevice);
    SetDtr (0);
    return (1);
  }

  return (0);
}


void
SetDtr (int hilo)
{
  DWORD dwFunc;

  if (hilo)
    dwFunc = SETDTR;
  else
    dwFunc = CLRDTR;

  if (EscapeCommFunction (hDevice, dwFunc) == 0) {
    ErrorString ();
  }
  return;
}


int
SetBaudRate (DWORD rate)
{

  Dcb.DCBlength = sizeof Dcb;
  GetCommState (hDevice, &Dcb);
  Dcb.BaudRate = rate;
  save_baud_rate = rate;
  //if (argc > 2)  {
  //    Dcb.BaudRate = atoi(argv[2]) ;
  //}
  if (!SetCommState (hDevice, &Dcb)) {
    ErrorString ();
    return (0);
  }
  return (1);
}


int
WriteSerialChar (unsigned char c)
{
  unsigned char buf[1];
  DWORD nb;

  buf[0] = c;
  nb = 1;
  if (WriteFile (hDevice, buf, 1, &nb, NULL) == 0) {
    ErrorString ();
    return (1);
  }
  return (0);
}


int
WaitTxBufferEmpty (void)
{
  DWORD lpEvtMask;

  lpEvtMask = EV_TXEMPTY;
  SetCommMask (hDevice, lpEvtMask);
  while (1) {
    if (WaitCommEvent (hDevice, &lpEvtMask, NULL) == 0) {
      ErrorString ();
      return (1);
    }
    if (lpEvtMask == EV_TXEMPTY)
      return (0);
  }
}




DWORD
ChkSerialInput (void)
{

//  get serial com port status
  bResult = ClearCommError (hDevice, &lpErrors, &lpStat);
  return lpStat.cbInQue;
}



int
ReadSerialChar (unsigned char *buf)
{

  DWORD nb;
// attempt a synchronous read operation
  bResult = ReadFile (hDevice, buf, 1, &nb, NULL);
  // check for eof
#if 0
  if (bResult && nb == 0) {
    // we're at the end of the file
    my_exit ("file EOF\n");
  }
#endif
  if (!bResult) {
    printf ("Bad read result\n");
    // deal with the error code
    FormatMessage (FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
                     NULL, GetLastError (), 
                     MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),      // Default language
                     (LPTSTR) & lpMsgBuf, 0, NULL);

    // Display the string.
    printf ("%s\n", lpMsgBuf);
    //            MessageBox( NULL, lpMsgBuf, "GetLastError", MB_OK|MB_ICONINFORMATION );

    // Free the buffer.
    LocalFree (lpMsgBuf);
    CloseHandle (hDevice);
    //goto again;
    OpenSerialPort (sCommName);
    SetBaudRate (save_baud_rate);
  }
  if (nb > 0) {
    return (1);
  }
  return (-1);
}

void
CleanUp (void)
{
  CloseHandle (hDevice);
}


void My_Get_Time( int *hour, int *min, int *sec, int *msec)
{
  struct time t;

        gettime(&t);
        *hour = t.ti_hour;
        *min = t.ti_min;
        *sec= t.ti_sec;
        *msec= t.ti_hund;
        return;
}

void scrn_init(void) {
  return;
}

int
my_delay(int MilliSeconds)
{
// wait for indicated mS
  Sleep(MilliSeconds);
  return (1);
}

int
my_getch(void)
{
  return getch();
}

void
my_refresh(void)
{
  return ;
}

void
refresh(void)
{
  return ;
}

void CloseConsole(void)
{
  return;
}


void
my_exit (char *exit_msg)
{
//   CleanUp ();
//   CloseConsole();
   printf("\r\n%s\r\n\r\n",exit_msg);
   exit (0);
}


void my_putchar(unsigned char ch)
{
  putchar(ch);
}

#endif  // WINDOWS



#ifdef  __linux__
/*********************************************************/
/*********************************************************/
/*********************************************************/
/*                    L I N U X   C O D E                */
/*********************************************************/
/*********************************************************/
/*********************************************************/

int console_open = 0;


int
OpenSerialPort (char *sCommPortName)
{
  int dummy;
  int file_descriptor;
  file_descriptor = OpenCOM(dummy, sCommPortName);
  return file_descriptor;
}


void
SetDtr (int hilo)
{
  if (hilo) Serial_SetDTR () ;
  else Serial_ClearDTR ();
  return;
}

int
SetBaudRate (DWORD rate)
{
 int dummyport;
 int new_baud;
 int result;

  new_baud = rate;
  result = SetBaudCOM(dummyport, new_baud);
  return result;
}

int
WriteSerialChar (unsigned char c)
{
 int dummyport;
 int outlen = 1;
 unsigned char outbuf [1];

  outbuf [0] = c;
  WriteCOM( dummyport, outlen, outbuf);

  return 0;
}

#if 0
int
WaitTxBufferEmpty (void)
{
}
#endif

int
ChkSerialInput (void)
{
 int dummyport;
   return TstSerialInput(dummyport);
}

int
ReadSerialChar (unsigned char *buf)
{
 int dummyport;
  int inlen = 1;
  int nread;
  nread = 0;
  ReadCOM(dummyport, inlen, buf, &nread);
  return 0;
}

void
CleanUp (void)
{
 int dummyport;
  CloseCOM(dummyport);
  return;
}


void My_Get_Time( int *hour, int *min, int *sec, int *msec)
{
  time_t curtime;
  struct tm *t;
  struct timeval tms;

    curtime = time(NULL);
    t = localtime(&curtime);
    *hour = (int) t->tm_hour;
    *min  = (int) t->tm_min;
    *sec  = (int) t->tm_sec;
    gettimeofday(&tms,NULL);
    *msec = (int) tms.tv_usec;
    return;
}

void gotoxy(int x, int y)
{
   move(y-1,x-1);
   refresh();
}

void scrn_init(void) {

   if (! OpenConsole()) {
     console_open = 0;
     my_exit("OpenConsole failed\r\n");
   }
   console_open = 1;
   clear();
   refresh();
   return;
}

void clrscr(void) {

   clear();
   refresh();
   return;
}
void clreol(void) {

   clrtoeol();
   refresh();
   return;
}

int
my_delay(int MilliSeconds)
{
// wait for indicated mS
  msDelay(MilliSeconds);
  return (1);
}

int
kbhit(void)
{
// Test for console port chars typed
//
// Returns 0, or 1
//
  fd_set         filedescr;
  struct timeval tval ;

    // set a descriptor to wait for a character available
    FD_ZERO(&filedescr);
    FD_SET(fileno(stdin),&filedescr);     // 0 is stdin
    /* set timeout to 100us */
    tval.tv_sec = 0;
    tval.tv_usec = 0;

    // if byte available read or return bytes read
    if (select(fileno(stdin)+1,&filedescr,NULL,NULL,&tval) == -1)  return 0;
    return 1;
}

int
my_getch(void)
{
  int ch;

  ch = getch();
  if (ch == 0x03)  // ^C
    {
      my_exit("Control-C exit");
    }
  else return ch;
  return 0;
}



void
my_exit (char *exit_msg)
{
  int inchar;

   //CleanUp ();
   printf("\r\n%s\r\n",exit_msg);
   if (!console_open) exit(0);
   printf("Press return to continue...\r\n");
   refresh();
   while (1) {
     my_delay(100);
     inchar = fgetc(stdin);  // must read into var because getc is a macro
     if (inchar == 0x0d) break;
   }
   CloseConsole();
   exit (0);
}


void cprintf( char* format, ...)
{
   va_list va;

   va_start(va, format);
   vprintf( format, va);
   va_end(va);
   refresh();
}



void my_putchar(unsigned char ch)
{
  addch(ch);
  refresh();
}

#endif /* LINUX */
